"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.GeoIP = exports.getLanguage = void 0;
const n8n_workflow_1 = require("n8n-workflow");
const maxmind_1 = __importDefault(require("maxmind"));
function getLanguage(lookupObject, language) {
    if (!lookupObject) {
        return '';
    }
    const languages = (lookupObject.names || {});
    return (languages[language] || languages['en']);
}
exports.getLanguage = getLanguage;
class GeoIP {
    constructor() {
        this.description = {
            displayName: 'GeoIP Node',
            name: 'geoIP',
            icon: 'fa:globe',
            group: ['transform'],
            version: 1,
            description: 'GeoIP node to lookup location or ASN information from an IP',
            defaults: {
                name: 'GeoIP',
            },
            inputs: ['main'],
            outputs: ['main'],
            properties: [
                {
                    displayName: 'Lookup Type',
                    name: 'lookupType',
                    type: 'options',
                    default: 'City',
                    options: [
                        {
                            name: "Location",
                            value: "City",
                        },
                        {
                            name: "ASN",
                            value: "ASN",
                        },
                    ],
                    description: 'The IP to Lookup',
                },
                {
                    displayName: 'IP',
                    name: 'ip',
                    type: 'string',
                    default: '',
                    placeholder: '1.1.1.1',
                    description: 'The IP to Lookup',
                },
                {
                    displayName: 'Simplify',
                    name: 'simplifyOutput',
                    type: 'boolean',
                    default: true,
                    description: 'Whether to return a simplified version of the response instead of the raw data',
                    displayOptions: {
                        hide: {
                            lookupType: ['ASN'],
                        },
                    },
                },
                {
                    displayName: 'Options',
                    name: 'options',
                    type: 'collection',
                    placeholder: 'Add Option',
                    default: {},
                    options: [
                        {
                            displayName: 'Put Output In Field',
                            name: 'outputField',
                            type: 'string',
                            default: '',
                            placeholder: 'geo',
                            description: 'The name of the output field to put the lookup data in',
                        },
                        {
                            displayName: 'Language',
                            name: 'language',
                            type: 'options',
                            displayOptions: {
                                show: {
                                    '/simplifyOutput': [true],
                                },
                            },
                            options: [
                                { name: 'Chinese', value: 'zh-CN' },
                                { name: 'English', value: 'en' },
                                { name: 'French', value: 'fr' },
                                { name: 'German', value: 'de' },
                                { name: 'Japansese', value: 'ja' },
                                { name: 'Portuguese', value: 'pt-BR' },
                                { name: 'Russian', value: 'ru' },
                                { name: 'Spanish', value: 'es' },
                            ],
                            default: 'en',
                            description: 'Language to display location names in',
                        },
                    ],
                },
            ],
        };
    }
    async execute() {
        const items = this.getInputData();
        let item;
        let ip;
        const lookupType = this.getNodeParameter('lookupType', 0, 'City');
        const simplifyOutput = this.getNodeParameter('simplifyOutput', 0, true);
        const outputField = this.getNodeParameter('options.outputField', 0, '');
        const language = this.getNodeParameter('options.language', 0, 'en');
        const { open, GeoIpDbName } = await import("geolite2-redist");
        const reader = await open(GeoIpDbName[lookupType], (path) => maxmind_1.default.open(path));
        for (let itemIndex = 0; itemIndex < items.length; itemIndex++) {
            try {
                ip = this.getNodeParameter('ip', itemIndex, '');
                item = items[itemIndex];
                let output = {};
                const res = reader.get(ip);
                if (!res) {
                    continue;
                }
                if (simplifyOutput && lookupType === "City") {
                    output.city = getLanguage(res.city, language);
                    output.country = getLanguage(res.country, language);
                    output.location = getLanguage(res.location, language);
                    output.continent = getLanguage(res.continent, language);
                    output.postal = (res.postal || {}).code;
                    output.registered_country = getLanguage(res.registered_country, language);
                    const latitude = (res.location || {}).latitude;
                    const longitude = (res.location || {}).longitude;
                    if (latitude && longitude) {
                        output.coordinates = `${latitude},${longitude}`;
                    }
                    output.subdivisions = (res.subdivisions || []).map(x => getLanguage(x, language));
                }
                else {
                    output = res;
                }
                if (outputField) {
                    item.json[outputField] = output;
                }
                else {
                    item.json = { ...item.json, ...output };
                }
            }
            catch (error) {
                if (this.continueOnFail()) {
                    items.push({ json: this.getInputData(itemIndex)[0].json, error, pairedItem: itemIndex });
                }
                else {
                    if (error.context) {
                        error.context.itemIndex = itemIndex;
                        throw error;
                    }
                    throw new n8n_workflow_1.NodeOperationError(this.getNode(), error, {
                        itemIndex,
                    });
                }
            }
        }
        return this.prepareOutputData(items);
    }
}
exports.GeoIP = GeoIP;
//# sourceMappingURL=GeoIP.node.js.map